(ns geocsv.core
  (:require
    [day8.re-frame.http-fx]
    [reagent.core :as r]
    [re-frame.core :as rf]
    [geocsv.gis :as gis]
    [geocsv.views.map :as mv]
    [goog.events :as events]
    [goog.history.EventType :as HistoryEventType]
    [markdown.core :refer [md->html]]
    [geocsv.ajax :as ajax]
    [geocsv.events]
    [reitit.core :as reitit]
    [reitit.frontend.easy :as rfe]
    [clojure.string :as s])
  (:import goog.History))

(defn nav-link [uri title page]
  [:a.navbar-item
   {:href   uri
    :class (when (= page @(rf/subscribe [:page])) :is-active)}
   title])

(defn navbar []
  (r/with-let [expanded? (r/atom false)]
    [:nav.navbar.is-info>div.container
     [:div.navbar-brand
      [:a.navbar-item {:href "/" :style {:font-weight :bold}} "geocsv"]
      [:span.navbar-burger.burger
       {:data-target :nav-menu
        :on-click #(swap! expanded? not)
        :class (when @expanded? :is-active)}
       [:span][:span][:span]]]
     [:div#nav-menu.navbar-menu
      {:class (when @expanded? :is-active)}
      [:div.navbar-start
       [nav-link "#/" "Home" :home]
       [nav-link "#/map" "Map" :map]
       [nav-link "#/about" "About" :about]]]]))

(defn about-page []
  [:section.section>div.container>div.content
   [:img {:src "/img/warning_clojure.png"}]
   (when-let [images @(rf/subscribe [:available-pin-images])]
     [:div
      [:h2 "The following pin images are available on this server"]
      (apply
        vector
        (cons
          :ol
          (map
            #(vector
               :ol
               [:img
                {:src
                 (str
                   "img/map-pins/"
                   (s/capitalize
                   (s/replace
                     (s/lower-case
                       (str %))
                     #"[^a-z0-9]" "-"))
                   "-pin.png")
                   :alt %}]
               " "
               %)
            (sort images))))])])

(defn home-page []
  [:section.section>div.container>div.content
   (when-let [docs @(rf/subscribe [:docs])]
     [:div {:dangerouslySetInnerHTML {:__html (md->html docs)}}])])

(defn map-page []
    "Return the content for the main map page. Map showing current location."
  (js/console.log "Attempting to switch to map: 1")
  (mv/panel))

(def pages
  {:home #'home-page
   :map #'map-page
   :about #'about-page})

(defn page []
  (if-let [page @(rf/subscribe [:page])]
    [:div
     [navbar]
     [page]]))

(defn navigate! [match _]
  (rf/dispatch [:navigate match]))

(def router
  (reitit/router
    [["/" {:name        :home
           :view        #'home-page
           :controllers [{:start (fn [_] (rf/dispatch [:init-home]))}]}]
     ["/map" {:name        :map
           :view        #'map-page
           :controllers [{:start (fn [_] (rf/dispatch [:init-map]))}]}]
     ["/about" {:name :about
                :view #'about-page}]]))

(defn start-router! []
  (rfe/start!
    router
    navigate!
    {}))

;; -------------------------
;; Initialize app
(defn mount-components []
  (rf/clear-subscription-cache!)
  (r/render [#'page] (.getElementById js/document "app")))

(defn init! []
  (rf/dispatch-sync [:initialise-db])
  (rf/dispatch [:fetch-pin-image-names])
  (start-router!)
  (ajax/load-interceptors!)
  (mount-components))
